/***************************************************************************
 *   Copyright (C) 2010 by Dominik Seichter                                *
 *   domseichter@web.de                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 *   In addition, as a special exception, the copyright holders give       *
 *   permission to link the code of portions of this program with the      *
 *   OpenSSL library under certain conditions as described in each         *
 *   individual source file, and distribute linked combinations            *
 *   including the two.                                                    *
 *   You must obey the GNU General Public License in all respects          *
 *   for all of the code used other than OpenSSL.  If you modify           *
 *   file(s) with this exception, you may extend this exception to your    *
 *   version of the file(s), but you are not obligated to do so.  If you   *
 *   do not wish to do so, delete this exception statement from your       *
 *   version.  If you delete this exception statement from all source      *
 *   files in the program, then also delete it here.                       *
 ***************************************************************************/

#include "PdfEncodingObjectFactory.h"

#include "base/PdfDefinesPrivate.h"

#include "base/PdfEncodingFactory.h"
#include "base/PdfObject.h"
#include "base/PdfVecObjects.h"

#include "PdfDifferenceEncoding.h"
#include "PdfIdentityEncoding.h"
#include "PdfCMapEncoding.h"

//For temporary purpose

#include <iostream>
using namespace std;

namespace PoDoFo
{

const PdfEncoding *PdfEncodingObjectFactory::CreateEncoding (PdfObject *pObject, PdfObject *pToUnicode, bool bExplicitNames)
{
    if (pObject->IsReference ())
    {
        // resolve any references
        pObject = pObject->GetOwner ()->MustGetObject (pObject->GetReference ());
    }

    if (pObject->IsName ())
    {
        const PdfName & rName = pObject->GetName ();
        if (rName == PdfName ("WinAnsiEncoding"))
            return PdfEncodingFactory::GlobalWinAnsiEncodingInstance ();
        else if (rName == PdfName ("MacRomanEncoding"))
            return PdfEncodingFactory::GlobalMacRomanEncodingInstance ();
        else if (rName == PdfName ("StandardEncoding"))	// OC 13.08.2010
            return PdfEncodingFactory::GlobalStandardEncodingInstance ();
        else if (rName == PdfName ("MacExpertEncoding"))	// OC 13.08.2010 TODO solved
            return PdfEncodingFactory::GlobalMacExpertEncodingInstance ();
        else if (rName == PdfName ("SymbolEncoding"))	// OC 13.08.2010
            return PdfEncodingFactory::GlobalSymbolEncodingInstance ();
        else if (rName == PdfName ("SymbolSetEncoding"))	// CB 21.02.2019
            return PdfEncodingFactory::GlobalSymbolEncodingInstance ();
        else if (rName == PdfName ("ZapfDingbatsEncoding"))	// OC 13.08.2010
            return PdfEncodingFactory::GlobalZapfDingbatsEncodingInstance ();
        else if (rName == PdfName ("Identity-H"))
            return new PdfIdentityEncoding (0, 0xffff, true, pToUnicode);
    }
  	else if (pObject->HasStream ())
    {
		return new PdfCMapEncoding(pObject, pToUnicode);
    }

  	else if (pObject->IsDictionary ())
    {

		return new PdfDifferenceEncoding (pObject, true, bExplicitNames);
    }
    
    
    PODOFO_RAISE_ERROR_INFO (ePdfError_InternalLogic,
                             "Unsupported encoding detected!");

    //return NULL; Unreachable code
}

};				/* namespace PoDoFo */
